﻿using System;

namespace Funktionsdelegaten
{
    class Program
    {
        delegate int Add(int a, int b);

        static void Main()
        {
            EventDemo();

            AddWithDelegate(DoAdd);

            AddWithFunc(DoAdd);

            AddWithDelegate((m, n) => m + n);

            AddWithDelegate((s, t) =>
                    {
                        return s + t;
                    }
                );

            AddWithFunc((m, n) => m + n);

            AddWithFunc((s, t) =>
                    {
                        return s + t;
                    }
                );

            Console.ReadLine();
        }

        static int DoAdd(int x, int y)
        {
            return x + y;
        }

        static void AddWithDelegate(Add addMethod)
        {
            int result1 = addMethod(2, 3);
            Console.WriteLine(result1);

            int result2 = addMethod(4, 6);
            Console.WriteLine(result2);

            Console.WriteLine();
        }

        static void AddWithFunc(Func<int, int, int> addMethod)
        {
            int result1 = addMethod(2, 3);
            Console.WriteLine(result1);

            int result2 = addMethod(4, 6);
            Console.WriteLine(result2);

            Console.WriteLine();
        }

        #region EventDemo

        private static void EventDemo()
        {
            MyEventClass myEventClass = new MyEventClass();

            myEventClass.MyEvent += OnEvent;

            myEventClass.RaiseEvent();

            myEventClass.MyEvent -= OnEvent;

            Console.WriteLine();            
        }

        private static void OnEvent(Object sender, EventArgs e)
        {
            Console.WriteLine("Event raised.");
        }

        #endregion
    }
}